# -*- encoding: utf-8 -*-
# Module iadftmatrix

from numpy import *

def iadftmatrix(N):
    x = arange(N).reshape(N,1)
    u = x
    Wn = exp(-1j*2*pi/N)
    A = (1./sqrt(N)) * (Wn ** dot(u, x.T))
    return A




# -*- encoding: utf-8 -*-
# Module iainterpollin

def iainterpollin(f, pts):
    # f - one, two or three dimention array
    # pts - array of points to interpolate:
    # throws IndexError if there are indices out of range in pts
    from iainterpollin import iainterpollin1D, iainterpollin2D, iainterpollin3D
    import numpy as np
    f = f.astype(float)
    if f.ndim == 1:
        return iainterpollin1D(f, np.ravel(pts))
    if f.ndim == 2:
        return iainterpollin2D(f, pts)
    if f.ndim == 3:
        return iainterpollin3D(f, pts)

def iainterpollin1D(f, pts):
    # f - one dimention array
    # pts - array of points to interpolate:
    # throws IndexError if there are indices out of range in pts
    import numpy as np

    # integer indices
    ipts = np.floor(pts).astype(int)

    # fractional indices
    fpts = pts - ipts

    # workaround for the case some index equals the last valid index
    fpts[ipts>=f.shape[0]-1] += 1
    ipts[ipts>=f.shape[0]-1] -= 1

    I = ipts.copy()
    Ix = ipts.copy()+1

    a = f[I]
    b = f[Ix] - a

    return a + b*fpts


def iainterpollin2D(f, pts):
    # f - bi-dimentional image
    # pts - array (2xN) of points to interpolate:
    # throws IndexError if there are indices out of range in pts
    # let's consider x the first and y the second dimention
    import numpy as np

    # integer indices
    ipts = np.floor(pts).astype(int)

    # fractional indices
    fpts = pts - ipts

    # workaround for the case some index equals the last valid index in the respective dimension
    fpts[0][ipts[0]>=f.shape[0]-1] += 1
    ipts[0][ipts[0]>=f.shape[0]-1] -= 1
    fpts[1][ipts[1]>=f.shape[1]-1] += 1
    ipts[1][ipts[1]>=f.shape[1]-1] -= 1

    I = ipts.copy()
    Ix = ipts.copy()
    Ix[0] += 1
    Iy = ipts.copy()
    Iy[1] += 1
    Ixy = Ix.copy()
    Ixy[1] += 1

    a = f[I[0], I[1]]
    b = f[Ix[0], Ix[1]] - a
    c = f[Iy[0], Iy[1]] - a
    e = f[Ixy[0], Ixy[1]] - a - b - c

    return a + b*fpts[0] + c*fpts[1] + e*fpts[0]*fpts[1]


def iainterpollin3D(ff, pts):
    # ff - tri dimentional image
    # pts - array (3xN) of points to interpolate:
    # throws IndexError if there are indices out of range in pts
    # let's consider x the first dimension, y the second and z the third
    import numpy as np

    # integer indices
    ipts = np.floor(pts).astype(int)

    # fractional indices
    fpts = pts - ipts

    # workaround for the case some index equals the last valid index in the respective dimension
    fpts[0][ipts[0]>=ff.shape[0]-1] += 1
    ipts[0][ipts[0]>=ff.shape[0]-1] -= 1
    fpts[1][ipts[1]>=ff.shape[1]-1] += 1
    ipts[1][ipts[1]>=ff.shape[1]-1] -= 1
    fpts[2][ipts[2]>=ff.shape[2]-1] += 1
    ipts[2][ipts[2]>=ff.shape[2]-1] -= 1

    I = ipts.copy()
    Ix = ipts.copy()
    Ix[0] += 1
    Iy = ipts.copy()
    Iy[1] += 1
    Iz = ipts.copy()
    Iz[2] += 1
    Ixy = Ix.copy()
    Ixy[1] += 1
    Ixz = Ix.copy()
    Ixz[2] += 1
    Iyz = Iy.copy()
    Iyz[2] += 1
    Ixyz = Ixy.copy()
    Ixyz[2] += 1

    a = ff[I[0], I[1], I[2]]
    b = ff[Ix[0], Ix[1], Ix[2]] - a
    c = ff[Iy[0], Iy[1], Iy[2]] - a
    d = ff[Iz[0], Iz[1], Iz[2]] - a
    e = ff[Ixy[0], Ixy[1], Ixy[2]] - a - b - c
    f = ff[Ixz[0], Ixz[1], Ixz[2]] - a - b - d
    g = ff[Iyz[0], Iyz[1], Iyz[2]] - a - c - d
    h = ff[Ixyz[0], Ixyz[1], Ixyz[2]] - a - b - c - d - e - f - g

    return a + b*fpts[0] + c*fpts[1] + d*fpts[2] + e*fpts[0]*fpts[1] + f*fpts[0]*fpts[2] + g*fpts[1]*fpts[2] + h*fpts[0]*fpts[1]*fpts[2]




# -*- encoding: utf-8 -*-
# Module iahaarmatrix

from numpy import *

def iahaarmatrix(N):
    from iameshgrid import iameshgrid

    n = floor(log(N)/log(2))

    if 2**n != N:
       raise Exception, 'error: size '+str(N)+' is not multiple of power of 2'

    z, k = iameshgrid(1.*arange(N)/N, 1.*arange(N))
    p  = floor(log(maximum(1,k))/log(2))
    q  = k - (2**p) + 1
    z1 = (q-1)   / (2**p)
    z2 = (q-0.5) / (2**p)
    z3 = q       / (2**p)
    A  = (1/sqrt(N)) * ((( 2**(p/2.)) * ((z >= z1) & (z < z2))) \
                              + ((-2**(p/2.)) * ((z >= z2) & (z < z3))))
    A[0,:] = 1/sqrt(N)
    return A




# -*- encoding: utf-8 -*-
# Module iagshow

import numpy as np

def iagshow(X, X1=None, X2=None, X3=None, X4=None, X5=None, X6=None):

    if X.dtype == np.bool: X = np.where(X,255,0).astype('uint8')
    r = X
    g = X
    b = X
    if X1 is not None: # red 1 0 0
      if (X1.dtype != np.bool): raise Exception,'X1 must be binary overlay'
      r = np.where(X1,255,r)
      g = np.where(~X1,g,0)
      b = np.where(~X1,b,0)
    if X2 is not None: # green 0 1 0
      if (X2.dtype != np.bool): raise Exception,'X2 must be binary overlay'
      r = np.where(~X2,r,0)
      g = np.where(X2,255,g)
      b = np.where(~X2,b,0)
    if X3 is not None: # blue 0 0 1
      if (X3.dtype != np.bool): raise Exception,'X3 must be binary overlay'
      r = np.where(~X3,r,0)
      g = np.where(~X3,g,0)
      b = np.where(X3,255,b)
    if X4 is not None: # magenta 1 0 1
      if (X4.dtype != np.bool): raise Exception,'X4 must be binary overlay'
      r = np.where(X4,255,r)
      g = np.where(~X4,g,0)
      b = np.where(X4,255,b)
    if X5 is not None: # yellow 1 1 0
      if (X5.dtype != np.bool): raise Exception,'X5 must be binary overlay'
      r = np.where(X5,255,r)
      g = np.where(X5,255,g)
      b = np.where(~X5,b,0)
    if X6 is not None: # cyan 0 1 1
      if (X6.dtype != np.bool): raise Exception,'X6 must be binary overlay'
      r = np.where(~X6,r,0)
      g = np.where(X6,255,g)
      b = np.where(X6,255,b)
    return np.array([r,g,b])




# -*- encoding: utf-8 -*-
# Module iabwlp

from numpy import *
import string

def iabwlp(fsize, tc, n, option='circle'):
    from iafftshift import iafftshift

    def test_exp(x, y):
        try:
            return x**(2*y)
        except:
            return 1E300 # Infinito!

    rows, cols = fsize[0], fsize[1]
    mh, mw = rows/2, cols/2
    rr, cc = meshgrid(arange(-mh,rows-mh), arange(-mw,cols-mw), indexing='ij') # center
    if string.find(string.upper(option), 'SQUARE') != -1:
        H = 1./(1.+(sqrt(2)-1)*(maximum(abs(1.*rr/rows) , abs(1.*cc/cols))*tc)**(2*n))
    else:
        aux1 = ravel(sqrt(((1.*rr)/rows)**2 + ((1.*cc)/cols)**2)*tc)
        aux2 = 0.*aux1 + n
        aux = reshape(map(test_exp, aux1, aux2), cc.shape)
        H = 1./(1+(sqrt(2)-1)*aux)
    H = iafftshift(H)
    return H




# -*- encoding: utf-8 -*-
# Module iawcorr

def iawcorr(im1, im2, w):
    import numpy as np
    w = w.astype(float)
    wf = w.ravel()
    sw = np.sum(wf)
    im1f = im1.astype(float).ravel()
    im2f = im2.astype(float).ravel()
    mim1f = np.sum(im1f * wf)/sw
    mim2f = np.sum(im2f * wf)/sw

    im1x = im1f - mim1f
    im2x = im2f - mim2f

    cov12 = np.sum(im1x*im2x*wf)/sw
    cov11 = np.sum(im1x*im1x*wf)/sw
    cov22 = np.sum(im2x*im2x*wf)/sw

    return cov12/np.sqrt(cov11*cov22)




# -*- encoding: utf-8 -*-
# Module iacomb

from numpy import *

def iacomb(s, delta, offset):

    s = asarray(s)
    if product(s.shape) == 1:
        g = zeros(s)
        g[offset::delta] = 1
    elif s.size >= 2:
        g = zeros((s[0], s[1]))
        g[offset[0]::delta[0], offset[1]::delta[1]] = 1

    if s.size == 3:
        aux = zeros(s)
        for i in range(offset[2], s[2], delta[2]):
            aux[:,:,i] = g
        g = aux
    return g




# -*- encoding: utf-8 -*-
# Module iaotsu

from numpy import *

def iaotsu(f):
    n = product(shape(f))
    h = 1.*bincount(f.ravel()) / n  # used bincount instead of iahistogram
    if len(h) == 1: return 1,1
    x = arange(product(shape(h)))
    w0 = cumsum(h)
    w1 = 1 - w0
    eps = 1e-10
    m0 = cumsum(x * h) / (w0 + eps)
    mt = m0[-1]
    m1 = (mt - m0[0:-1]*w0[0:-1]) / w1[0:-1]
    sB2 = w0[0:-1] * w1[0:-1] * ((m0[0:-1] - m1)**2)
    t = argmax(sB2)
    v = sB2[t]
    st2 = sum((x-mt)**2 * h)
    eta = v / st2
    return t, eta




# -*- encoding: utf-8 -*-
# Module iapercentile

def iapercentile(f, p=1):

   import numpy as np
   k = (f.size-1) * p/100.
   dw = np.floor(k).astype(int)
   up = np.ceil(k).astype(int)
   g  = np.sort(f.ravel())
   d  = g[dw]
   d0 =   d   * (up-k)
   d1 = g[up] * (k -dw)
   return np.where(dw==up, d, d0+d1)




# -*- encoding: utf-8 -*-
# Module iapconv2

def iapconv2(f,h):
   import numpy as np

   h_ind=np.nonzero(h)
   f_ind=np.nonzero(f)
   if len(h_ind[0])>len(f_ind[0]):
       h,    f    = f,    h
       h_ind,f_ind= f_ind,h_ind

   gs = np.maximum(np.array(f.shape),np.array(h.shape))
   if (f.dtype == 'complex') or (h.dtype == 'complex'):
       g = np.zeros(gs,dtype='complex')
   else:
       g = np.zeros(gs)

   f1 = g.copy()
   f1[f_ind]=f[f_ind]

   if f.ndim == 1:
     (W,) = gs
     col = np.arange(W)
     for cc in h_ind[0]:
        g[:] += f1[(col-cc)%W] * h[cc]

   elif f.ndim == 2:
     H,W = gs
     row,col = np.indices(gs)
     for rr,cc in np.transpose(h_ind):
        g[:] += f1[(row-rr)%H, (col-cc)%W] * h[rr,cc]

   else:
     Z,H,W = gs
     d,row,col = np.indices(gs)
     for dd,rr,cc in np.transpose(h_ind):
        g[:] += f1[(d-dd)%Z, (row-rr)%H, (col-cc)%W] * h[dd,rr,cc]
   return g




# -*- encoding: utf-8 -*-
# Module iacos

import numpy as np

def iacos(s, t, theta, phi):
    r, c = np.indices(s)
    tc = t / np.cos(theta)
    tr = t / np.sin(theta)
    f = np.cos(2*np.pi*(r/tr + c/tc) + phi)
    return f




# -*- encoding: utf-8 -*-
# Module iaihwt

from numpy import *

def iaihwt(f):
    from iahaarmatrix import iahaarmatrix

    f = asarray(f).astype(float64)
    if len(f.shape) == 1: f = f[:,newaxis]
    (m, n) = f.shape
    A = iahaarmatrix(m)
    if (n == 1):
        F = dot(transpose(A), f)
    else:
        B = iahaarmatrix(n)
        F = dot(dot(transpose(A), f), B)
    return F




# -*- encoding: utf-8 -*-
# Module iaphasecorr

from numpy import *

def iaphasecorr(f,h):
    F = fft.fftn(f)
    H = fft.fftn(h)
    T = F*conjugate(H)
    R = T/abs(T)
    g = fft.ifftn(R)
    return g.real




# -*- encoding: utf-8 -*-
# Module iadctmatrix

from numpy import *

def iadctmatrix(N):
    from iameshgrid import iameshgrid
    x, u = iameshgrid(range(N), range(N)) # (u,x)
    alpha = ones((N,N)) * sqrt(2./N)
    alpha[0,:] = sqrt(1./N) # alpha(u,x)
    A = alpha * cos((2*x+1)*u*pi / (2.*N)) # Cn(u,x)
    return A




# -*- encoding: utf-8 -*-
# Module iacolormap

from numpy import *
import colorsys

def iacolormap(type='gray'):
    from ianormalize import ianormalize
    from iacolormap import iacolormap

    if type == 'gray':
        ct = transpose(resize(arange(256), (3,256)))
    elif type == 'hsv':
        h = arange(256)/255.
        s = ones(256)
        v = ones(256)
        ct = ianormalize(reshape(map(colorsys.hsv_to_rgb, h, s, v), (256,3)), [0,255]).astype(uint8)
    elif type == 'hot':
        n = int(3./8*256)
        r = concatenate((arange(1,n+1)/n, ones(256-n)), 1)[:,newaxis]
        g = concatenate((zeros(n), arange(1,n+1)/n, ones(256-2*n)), 1)[:,newaxis]
        b = concatenate((zeros(2*n), arange(1,256-2*n+1)/(256-2*n)), 1)[:,newaxis]
        ct = ianormalize(concatenate((r,g,b), 1), [0,255]).astype(uint8)
    elif type == 'cool':
        r = (arange(256)/255.)[:,newaxis]
        ct = ianormalize(concatenate((r, 1-r, ones((256,1))), 1), [0,255]).astype(uint8)
    elif type == 'bone':
        ct = ianormalize((7*iacolormap('gray') + iacolormap('hot')[:,::-1]) / 8., [0,255]).astype(uint8)
    elif type == 'copper':
        ct = ianormalize(min(1, dot(iacolormap('gray')/255., [[1.25,0,0],[0,0.7812,0],[0,0,0.4975]])), [0,255]).astype(uint8)
    elif type == 'pink':
        ct = ianormalize(sqrt((2*iacolormap('gray') + iacolormap('hot')) / 3), [0,255]).astype(uint8)
    else:
        ct = zeros((256,3))
    return ct




# -*- encoding: utf-8 -*-
# Module iadft

from numpy import *

def iadft(f):
    from ia636 import iadftmatrix
    f = asarray(f).astype(float64)
    if (len(f.shape) == 1):
        m = len(f)
        A = iadftmatrix(f.shape[0])
        F = sqrt(m) * dot(A, f)
    elif (len(f.shape) == 2):
        (m, n) = f.shape
        A = iadftmatrix(m)
        B = iadftmatrix(n)
        F = sqrt(m * n) * dot(dot(A, f), B)
    else:
        (p,m,n) = f.shape
        A = iadftmatrix(m)
        B = iadftmatrix(n)
        C = iadftmatrix(p)
        Faux = dot(A,f)
        Faux = dot(Faux,B)
        F = sqrt(p)*sqrt(m)*sqrt(n)*dot(C,Faux)
    return F




# -*- encoding: utf-8 -*-
# Module iahadamardmatrix

from numpy import *

def iahadamardmatrix(N):
    from iameshgrid import iameshgrid

    def bitsum(x):
        s = 0 * x
        while x.any():
            s += x & 1
            x >>= 1
        return s

    n = floor(log(N)/log(2))

    if 2**n != N:
       raise Exception, 'error: size '+str(N)+' is not multiple of power of 2'

    u, x = iameshgrid(range(N), range(N))

    A = ((-1)**(bitsum(x & u)))/sqrt(N)
    return A




# -*- encoding: utf-8 -*-
# Module iacircle

from numpy import *

def iacircle(s, r, c):

    rows, cols = s[0], s[1]
    rr0,  cc0  = c[0], c[1]
    rr, cc = meshgrid(range(rows), range(cols), indexing='ij')
    g = (rr - rr0)**2 + (cc - cc0)**2 <= r**2
    return g




# -*- encoding: utf-8 -*-
# Module iacrop

from numpy import *

def iacrop(f, side='all', color='black'):
    from ianeg import ianeg


    f = asarray(f)
    if len(f.shape) == 1: f = f[newaxis,:]
    if color == 'white': f = ianeg(f)

    aux1, aux2 = sometrue(f,0), sometrue(f,1)
    col, row = flatnonzero(aux1), flatnonzero(aux2)

    #if (not col) and (not row):
    #    return None

    if   side == 'left':   g = f[:, col[0]::]
    elif side == 'right':  g = f[:, 0:col[-1]+1]
    elif side == 'top':    g = f[row[0]::, :]
    elif side == 'bottom': g = f[0:row[-1]+1, :]
    else:                  g = f[row[0]:row[-1]+1, col[0]:col[-1]+1]

    if color == 'white': g = ianeg(g)
    return g




# -*- encoding: utf-8 -*-
# Module iah2percentile

def iah2percentile(h,p):

    import numpy as np
    s = h.sum()
    k = ((s-1) * p/100.)+1
    dw = np.floor(k)
    up = np.ceil(k)
    hc = np.cumsum(h)
    if isinstance(p, int):
       k1 = np.argmax(hc>=dw)
       k2 = np.argmax(hc>=up)
    else:
       k1 = np.argmax(hc>=dw[:,np.newaxis],axis=1)
       k2 = np.argmax(hc>=up[:,np.newaxis],axis=1)
    d0 = k1 * (up-k)
    d1 = k2 * (k -dw)
    return np.where(dw==up,k1,d0+d1)




# -*- encoding: utf-8 -*-
# Module iapad

from numpy import *

def iapad(f, thick=[1,1], value=0):

    f, thick = asarray(f), asarray(thick)
    g = (value * ones(array(f.shape)+2*thick)).astype(f.dtype.char)
    g[thick[0]:-thick[0], thick[1]:-thick[1]] = f
    return g




# -*- encoding: utf-8 -*-
# Module iaplot

def iaplot(Ylist, Xlist = [], arrows_list = [],text_list = [], ylabel='y', xlabel='x', title='',colors = 'rgbmyc',shapes = '------', axis = 'tight', fig_size = [], face_color = 'w'):
    import numpy as np
    import matplotlib.pyplot as plt
    import ia636

    if isinstance(Ylist, np.ndarray):
        Ylist = [Ylist]
    if  Xlist == []:
       Xlist = [np.arange(len(i)) for i in Ylist]
    if isinstance(Xlist, np.ndarray):
        Xlist = [Xlist]
    if fig_size == [] :
        fig = plt.figure(facecolor = face_color)
    else:
        fig = plt.figure(figsize = fig_size, facecolor = face_color)


    plt.ylabel(ylabel)
    plt.xlabel(xlabel)
    plt.title(title)
    for x,y, c,s in zip(Xlist, Ylist,colors[:len(Xlist)],shapes[:len(Xlist)]):
        plt.plot(x,y, c+s,markersize=9)

    for arrow in arrows_list:
        plt.arrow(arrow[0], arrow[1], arrow[2], arrow[3], fc="k", ec="k", head_width=0.15, head_length=0.2 )

    for text in text_list:
        plt.annotate(text[0],xy=(text[1], text[2]), xycoords='data',xytext=(text[1], text[2]), textcoords='data',)
    plt.grid()
    plt.axis(axis)
    return ia636.iafig2img(fig)




# -*- encoding: utf-8 -*-
# Module iaisdftsym

from numpy import *

def iaisdftsym(F):

    if len(F.shape) == 1: F = F[newaxis,newaxis,:]
    if len(F.shape) == 2: F = F[newaxis,:,:]

    n,m,p = F.shape
    x,y,z = indices((n,m,p))

    Xnovo = mod(-1*x,n)
    Ynovo = mod(-1*y,m)
    Znovo = mod(-1*z,p)

    aux = conjugate(F[Xnovo,Ynovo,Znovo])

    return alltrue(abs(F-aux)<10E-4)




# -*- encoding: utf-8 -*-
# Module ianormalize

from numpy import *

def ianormalize(f, range=[0,255]):

    f = asarray(f)
    range = asarray(range)
    if f.dtype.char in ['D', 'F']:
        raise Exception, 'error: cannot normalize complex data'
    faux = ravel(f).astype(float)
    minimum = faux.min()
    maximum = faux.max()
    lower = range[0]
    upper = range[1]
    if upper == lower:
        g = ones(f.shape) * maximum
    if minimum == maximum:
        g = ones(f.shape) * (upper + lower) / 2.
    else:
        g = (faux-minimum)*(upper-lower) / (maximum-minimum) + lower
    g = reshape(g, f.shape)

    if f.dtype == uint8:
        if upper > 255:
            raise Exception,'ianormalize: warning, upper valuer larger than 255. Cannot fit in uint8 image'
    g = g.astype(f.dtype) # set data type of result the same as the input image
    return g




# -*- encoding: utf-8 -*-
# Module MainPage

#
# Toolbox ia636 (in alphabetical order)
#

__version__ = 'ia636 Python Toolbox for Teaching Image  Processing version 0.11.7 26th Aug 2014'

from iaapplylut import iaapplylut
from iabwlp import iabwlp
from iacircle import iacircle
from iacolorhist import iacolorhist
from iacolormap import iacolormap
from iacomb import iacomb
from iacontour import iacontour
from iaconv import iaconv
from iacorr import iacorr
from iacos import iacos
from iacrop import iacrop
from iadct import iadct
from iadctmatrix import iadctmatrix
from iadft import iadft
from iadftmatrix import iadftmatrix
from iadither import iadither
from iadftview import iadftview
from iaellipse import iaellipse
from iaffine import iaffine
from iaffine3 import iaffine3
from iafftshift import iafftshift
from iafig2img import iafig2img
from iafloyd import iafloyd
from iaftemplate import iaftemplate
from iagaussian import iagaussian
from iageorigid import iageorigid
from iagshow import iagshow
from iah2percentile import iah2percentile
from iah2stats import iah2stats
from iahaarmatrix import iahaarmatrix
from iahadamard import iahadamard
from iahadamardmatrix import iahadamardmatrix
from iahistogram import iahistogram, iahistogram_eq, iahistogram_eq1
from iahwt import iahwt
from iaidct import iaidct
from iaidft import iaidft
from iaifftshift import iaifftshift
from iaihadamard import iaihadamard
from iaihwt import iaihwt
from iaimginfo import iaimginfo
from iaind2sub import iaind2sub
from iainterpollin import iainterpollin
from iainterpolclosest import iainterpolclosest
from iaisdftsym import iaisdftsym
from iaisolines import iaisolines
from ialblshow import ialblshow
from ialog import ialog
from ialogfilter import ialogfilter
from iameshgrid import iameshgrid
from iamosaic import iamosaic
from ianeg import ianeg
from ianormalize import ianormalize
from ianshow import ianshow
from iaotsu import iaotsu
from iapad import iapad
from iapca import iapca
from iapconv import iapconv
from iapconv2 import iapconv2
from iapercentile import iapercentile
from iaphasecorr import iaphasecorr
from iaplot import iaplot
from iapolar import iapolar
from iaptrans import iaptrans
from iaramp import iaramp
from iareadurl import iareadurl
from iarectangle import iarectangle
from iaroi import iaroi
from iargb2gray import iargb2gray
from iarot90 import iarot90
from iasobel import iasobel
from iastat import iastat
from iasub2ind import iasub2ind
from iatcrgb2ind import iatcrgb2ind
from iatext import iatext
from iatile import iatile
from iatiling import iatiling
from iavarfilter import iavarfilter
from iawcorr import iawcorr



# -*- encoding: utf-8 -*-
# Module iaramp

from numpy import *

def iaramp(s, n, range=[0,255]):
    aux = array(n)
    s_orig = s

    if len(aux.shape) == 0:
        s = [1,s[0],s[1]]
        n = [0,0,n]
        range = [0,0,0,0,range[0],range[1]]

    slices,rows, cols = s[0], s[1], s[2]
    z,y,x = indices((slices,rows,cols))
    gz = z*n[0]/slices * (range[1]-range[0]) / (n[0]-1) + range[0]
    gy = y*n[1]/rows * (range[3]-range[2]) / (n[1]-1) + range[2]
    gx = x*n[2]/cols * (range[5]-range[4]) / (n[2]-1) + range[4]
    return (gz+gy+gx).reshape(s_orig)




# -*- encoding: utf-8 -*-
# Module iaconv

import numpy as np

def iaconv(f, h):
    f, h = np.asarray(f), np.asarray(h,float)
    if len(f.shape) == 1: f = f[np.newaxis,:]
    if len(h.shape) == 1: h = h[np.newaxis,:]
    if f.size < h.size:
        f, h = h, f
    g = np.zeros(np.array(f.shape) + np.array(h.shape) - 1)
    if f.ndim == 2:
        for i in xrange(h.shape[0]):
            for j in xrange(h.shape[1]):
                if h[i,j]:
                  g[i:i+f.shape[0], j:j+f.shape[1]] += h[i,j] * f

    if f.ndim == 3:
        for i in xrange(h.shape[0]):
            for j in xrange(h.shape[1]):
                for k in xrange(h.shape[2]):
                    if h[i,j,k]:
                      g[i:i+f.shape[0], j:j+f.shape[1], k:k+f.shape[2]] += h[i,j,k] * f

    return g




# -*- encoding: utf-8 -*-
# Module iaidft

from numpy import *

def iaidft(F):
    from ia636 import iadftmatrix

    s = F.shape
    if len(F.shape) == 1: F = F[newaxis,newaxis,:]
    if len(F.shape) == 2: F = F[newaxis,:,:]

    (p,m,n) = F.shape
    A = iadftmatrix(m)
    B = iadftmatrix(n)
    C = iadftmatrix(p)
    Faux = dot(conjugate(A),F)
    Faux = dot(Faux,conjugate(B))
    f = dot(conjugate(C),Faux)/(sqrt(p)*sqrt(m)*sqrt(n))

    return f.reshape(s)




# -*- encoding: utf-8 -*-
# Module iatcrgb2ind

import numpy as np

def iatcrgb2ind(f):

    f = np.asarray(f)
    r, g, b = f[0].astype(np.int), f[1].astype(np.int), f[2].astype(np.int)
    c = r + 256*g + 256*256*b

    (t,i) = np.unique(c,return_inverse=True)

    n = len(t)
    rt = np.reshape(map(lambda k:int(k%256), t), (n,1))
    gt = np.reshape(map(lambda k:int((k%(256*256))/256.), t), (n,1))
    bt = np.reshape(map(lambda k:int(k), t/(256*256.)), (n,1))

    cm = np.concatenate((rt, gt, bt), axis=1)
    fi = i.reshape(r.shape)
    return fi,cm




# -*- encoding: utf-8 -*-
# Module iapolar

from numpy import *

def iapolar(f, domain, thetamax = 2 * pi):

    from ia636 import iainterpollin
    f = array(f)
    m,n = f.shape
    dm,dn = domain
    Ry,Rx = floor(array(f.shape)/2)

    b = min(Ry,Rx)/dm
    a = thetamax/dn

    y,x = indices(domain)

    XI = Rx + (b*y)*cos(a*x)
    YI = Ry + (b*y)*sin(a*x)

    g = iainterpollin(f, array([YI.ravel(), XI.ravel()]))
    g.shape = domain

    return g




# -*- encoding: utf-8 -*-
# Module iacolorhist

def iacolorhist(f, mask=None):
    import numpy as np
    from iahistogram import iahistogram

    WFRAME=5
    f = np.asarray(f)
    if len(f.shape) == 1: f = f[np.newaxis,:]
    if not f.dtype == 'uint8':
      raise Exception,'error, can only process uint8 images'
    if not f.shape[0] == 3:
      raise Exception, 'error, can only process 3-band images'

    r,g,b = f[0].astype(np.int), f[1].astype(np.int), f[2].astype(np.int)

    n_zeros = 0
    if mask:
      n_zeros = f.shape[0]*f.shape[1]-len(np.nonzero(np.ravel(mask)))
      r,g,b = mask*r, mask*g, mask*b

    hrg = np.zeros((256,256), np.int32); hbg=hrg+0; hrb=hrg+0
    img = 256*r + g; m1 = img.max()
    aux = iahistogram(img.astype(np.int32)); aux[0] = aux[0] - n_zeros
    np.put(np.ravel(hrg), range(m1+1), aux)
    img = 256*b + g; m2 = img.max()
    aux = iahistogram(img.astype(np.int32)); aux[0] = aux[0] - n_zeros
    np.put(np.ravel(hbg), range(m2+1), aux)
    img = 256*r + b; m3 = img.max()
    aux = iahistogram(img.astype(np.int32)); aux[0] = aux[0] - n_zeros
    np.put(np.ravel(hrb), range(m3+1), aux)
    m=max(hrg.max(),hbg.max(),hrb.max())
    hc=m*np.ones((3*WFRAME+2*256,3*WFRAME+2*256))
    hc[WFRAME:WFRAME+256,WFRAME:WFRAME+256] = np.transpose(hrg)
    hc[WFRAME:WFRAME+256,2*WFRAME+256:2*WFRAME+512] = np.transpose(hbg)
    hc[2*WFRAME+256:2*WFRAME+512,WFRAME:WFRAME+256] = np.transpose(hrb)
    return hc




# -*- encoding: utf-8 -*-
# Module iadither

from numpy import *

def iadither(f, n):
    import ia636

    H,W = f.shape
    D = 1.*array([[0,2],[3,1]])
    d = 1*D
    k = int(log(n/2.)/log(2.))
    for i in range(k):
        u = ones(D.shape)
        d1 = 4*D + d[0,0]*u
        d2 = 4*D + d[0,1]*u
        d3 = 4*D + d[1,0]*u
        d4 = 4*D + d[1,1]*u
        D = concatenate((concatenate((d1,d2),1), concatenate((d3,d4),1)))
    D = (255*abs(D/D.max())).astype('uint8')
    g = tile(D, array(f.shape)//array(D.shape) + array([1,1]))[:H,:W]
    g = ia636.ianormalize(f,[0,255]) >= g
    return g




# -*- encoding: utf-8 -*-
# Module iavarfilter

from numpy import *

def iavarfilter(f,h,CV=True):

    from ia636 import iapconv

    f = asarray(f).astype(float64)
    h = asarray(h).astype(bool)[::-1,::-1]

    n = sum(ravel(h))
    h = h/float(n)
    fm = iapconv(f, h)
    f2m = iapconv(f**2, h)

    g = f2m - fm**2

    if (CV):
        fm = fm + 1e-320*(fm == 0) # change zero by a very small number (prevent 'math range error')
        g[g<0.0] = 0.0 # avoid negative numbers due to numeric precision
        g = sqrt(g) / fm

    return g




# -*- encoding: utf-8 -*-
# Module iaisolines

import numpy as np
import ia636 as ia

def iaisolines(f, nc=10, n=1):
    maxi = np.ceil(f.max())
    mini = np.floor(f.min())
    d = int(np.ceil(1.*(maxi-mini)/nc))
    m = np.zeros((d,1))
    m[0:n,:] = 1
    m = np.resize(m, (maxi-mini, 1))
    m = np.concatenate((np.zeros((mini,1)), m))
    m = np.concatenate((m, np.zeros((256-maxi,1))))
    m = np.concatenate((m,m,m), 1)
    ct = m*ia.iacolormap('hsv') + (1-m)*ia.iacolormap('gray')
    g = ia.iaapplylut(f, ct)
    return g




# -*- encoding: utf-8 -*-
# Module iaimginfo

import numpy as np

def iaimginfo(f):
    t = type(f)
    if t != np.ndarray:
        return 'Not a ndarray. It is %s' % (t,)
    else:
        dt = f.dtype
        if dt == 'bool':
            return '%s %s %s %s %s' % (t, np.shape(f), f.dtype, f.min(), f.max())
        elif dt == 'uint8':
            return '%s %s %s %d %d' % (t, np.shape(f), f.dtype, f.min(), f.max())
        else:
            return '%s %s %s %f %f' % (t, np.shape(f), f.dtype, f.min(), f.max())




# -*- encoding: utf-8 -*-
# Module iadftview

from numpy import *

def iadftview(F):
    from ia636 import iafftshift
    from ia636 import ianormalize

    FM = iafftshift(log(abs(F)+1))
    return ianormalize(FM).astype(uint8)

